"""fhs_enyaq_data - enyaq car data versions """

__version__ = "0.1.4"
__author__ = "Richard de Vos <rdevos72@gmail.com>"
__author_name__ = "Richard de Vos"
__author_email__ = "rdevos72@gmail.com"
__app_name__ = "fhs_enyaq_data"
__project_name__ = "fhs-enyaq-data"
__all__ = []



""" Loop. """
import time
from pprint import pprint

from fhs_enyaq_data.mqtt import mqtt_connect, mqtt_publish_instruments

def data_loop(idle_wait=15, drive_wait=5, charge_wait=5, output=print):
    last_km = None
    from .fhs_enyaq_data import get_instruments_with_timeout
    from .config import get_config
    from .abrp_send import send_abrp
    config = get_config()
    # username = config['mqtt']['username'],
    if 'mqtt' in config:
        output('mqtt start')
        mqtt = mqtt_connect(
            host = config['mqtt']['host'],
            port = config['mqtt']['port'],
            tls = config['mqtt'].get('tls', False),
            username = config['mqtt'].get('username', None),
            password = config['mqtt'].get('password', None),
            info_output = output,
        )
    else:
        mqtt = None

    while True:
        # run
        output('get instruments information from skoda connect.')
        instruments = get_instruments_with_timeout(config)
        if instruments is not None:
            # pprint(instruments)
            if 'Battery level' not in instruments:
                output('no battery level returned, sleeping 30 seconds.')
                time.sleep(30)
                continue
            output(f"battery level: {instruments['Battery level']}   charging: {instruments['Charging']}")
            send_abrp(config, instruments, output=output)
            if mqtt is not None:
                mqtt_publish_instruments(mqtt, config['mqtt']['topic'], instruments)
                #pprint(instruments)
            sleep_time = idle_wait * 60
            if last_km is None:
                last_km = instruments['Electric range']
            if instruments['Charging'] == 1:
                sleep_time = charge_wait * 60
                output('charging.')
            elif last_km != instruments['Electric range']:
                sleep_time = drive_wait * 60
                output('driving.')
            else:
                output('parked or just starting to drive.')
            last_km = instruments['Electric range']
            output(f"going to sleep for {sleep_time} seconds.")
            time.sleep(sleep_time)
        else:
            output('no instruments returned, sleeping 120 seconds.')
            time.sleep(120)





"""Console script for fhs_enyaq_data."""
import sys

import typer
from pprint import pprint
main = typer.Typer()

@main.command()
def list_car_types():
    from .abrp_list_cars import list_cars
    result = list_cars()
    for k in result:
        print(f"{k:40}  {result[k]}")


@main.command()
def get_data(
        vehicle_vin: str = typer.Option("", help="Extra help")
):
    from .fhs_enyaq_data import get_vehicle_data
    get_vehicle_data()

    return 0


@main.command()
def get_instruments(
):
    from .fhs_enyaq_data import print_instruments
    from .config import get_config
    config = get_config()
    print_instruments(config)

    return 0

@main.command()
def send_data(
        vehicle_vin: str = typer.Option("", help="Extra help"),
):
    from .fhs_enyaq_data import get_instruments
    from .config import get_config
    from .abrp_send import send_abrp
    config = get_config()
    instruments = get_instruments(config)
    pprint(instruments)
    send_abrp(config, instruments)

    return 0

@main.command()
def send_data_loop(
        idle_wait: int = typer.Option(10,  help="delay in idle wait"),
        drive_wait: int = typer.Option(5,  help="delay in drivemode wait"),
        charge_wait: int = typer.Option(5,  help="delay in charge wait"),
):
    from .loop import data_loop
    from .output import console_str
    data_loop(idle_wait, drive_wait, charge_wait, output=console_str)

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



"""Main module."""

from fhs_enyaq_data.skoda_class.skoda_class import skoda_class
import signal

def main(config):
    """ Skoda. """
    skoda = skoda_class(config=config['skoda'], verbose=False)
    skoda.list_cars()


def get_vehicle_data(config):
    if 'vehicle_vin' not in config['skoda'] or config['skoda']['vehicle_vin'] == '':
        vehicle_vin = None
    else:
        vehicle_vin = config['skoda']['vehicle_vin']
    result = skoda.get_battery_level(vehicle_vin)
    skoda = skoda_class(config=config['skoda'])
    print(f"{result=}")


def print_instruments(config):
    if 'vehicle_vin' not in config['skoda'] or config['skoda']['vehicle_vin'] == '':
        vehicle_vin = None
    else:
        vehicle_vin = config['skoda']['vehicle_vin']
    skoda = skoda_class(config=config['skoda'])
    result = skoda.print_instruments(vehicle_vin)

def get_instruments(config):
    if 'vehicle_vin' not in config['skoda'] or config['skoda']['vehicle_vin'] == '':
        vehicle_vin = None
    else:
        vehicle_vin = config['skoda']['vehicle_vin']
    skoda = skoda_class(config=config['skoda'])
    result = skoda.get_instruments(vehicle_vin)
    return (result)

def get_instruments_with_timeout(config, timeout=60):
    class TimeOutException(Exception):
        pass

    def alarm_handler(signum, frame):
        raise TimeOutException()

    signal.signal(signal.SIGALRM, alarm_handler)
    signal.alarm(timeout)

    try:
        result = get_instruments(config)
    except TimeOutException as ex:
        result = None
    signal.alarm(0)
    return result




"""Send abrp."""

from .abrp.abrp_class import abrp_class
from pprint import pprint

def send_abrp(config, data, output=None):
    abrp = abrp_class(token=config['abrp']['token'], car_type=config['abrp'].get('car_type', "skoda:enyaq:21:77:meb"))
    result = abrp.send_data(data)
    if output is not None:
        output(f"abrp send, result: {result}")
        # output(f"abrp content, result: {result.content}")
        # output(f"abrp url, result: {result.url}")
    return result



"""list abrp."""

from .abrp.abrp_class import abrp_class
from pprint import pprint

def list_cars():
    car_types={}
    abrp = abrp_class(token="")
    result = abrp.get_car_types()
    if result is None:
        print('error getting car types.')
        exit(1)
    for i in result['result']:
        for k in i:
            if 'ENYAQ' in k.upper():
                car_types[k] = i[k]
    return car_types



import io
import os
import re

from setuptools import find_packages
from setuptools import setup


about = {}
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'src', 'fhs_enyaq_data', '__version__.py')) as f:
    exec(f.read(), about)


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding='utf-8') as fd:
        return re.sub(text_type(r':[a-z]+:`~?(.*?)`'), text_type(r'``\1``'), fd.read())


with open("./requirements.txt") as requirements:
    REQUIREMENTS = requirements.read().splitlines()


setup(
    name="fhs-enyaq-data",
    version=about["__version__"],
    url="https://github.com/foxhunt72/fhs-enyaq-data",
    license="MIT license",
    author="Richard de Vos",
    author_email="rdevos72@gmail.com",
    description="enyaq car data",
    long_description=read("README.rst"),
    entry_points={
        "console_scripts": [
            "fhs-enyaq-data=fhs_enyaq_data.cli:main",
        ],
    },

    packages=find_packages(
        where="src",
        exclude=(
            "tests",
            "tests_*",
        )
    ),
    package_dir={"": "src"},
    install_requires=REQUIREMENTS,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)

